#### R scipt for implementing our simulation to compare the standard likelihood approach
#### with the proposed methodology
#### aAcompanies the paper Bilton et al. (2017) Linkage disequilibrium estimation in low coverage sequencing data.
## Author: Timothy Bilton
## Date: 18/10/17

## create subfolder for the simulated data to go
system("mkdir SimData")
library(boot)

### Function for estimating D' and r2 using the standard approach without adjusting for depth
estMLE <- function(GBSgF,GBSfreq){
  if(any(GBSfreq %in% c(0,1))){ return(c(0,NA,NA))  }
  else{
    n11 <- GBSgF[1,1]
    n12 <- GBSgF[1,2]
    n13 <- GBSgF[1,3]
    n21 <- GBSgF[2,1]
    n22 <- GBSgF[2,2]
    n23 <- GBSgF[2,3]
    n31 <- GBSgF[3,1]
    n32 <- GBSgF[3,2]
    n33 <- GBSgF[3,3]
    N <- sum(GBSgF)
    pa1 = GBSfreq[1]; pa2 = GBSfreq[2]
    
    # Likelihood function
    ll <- function(D,pA1,pA2){
      pB1 = 1-pA1
      pB2 = 1-pA2
      -((2*n11+n12+n21)*log(pA1*pA2+D) + (2*n13+n12+n23)*log(pA1*pB2-D) + 
          (2*n31+n21+n32)*log(pB1*pA2-D) + (2*n33+n32+n23)*log(pB1*pB2+D) + 
          (n22)*log((pA1*pA2+D)*(pB1*pB2+D)+(pA1*pB2-D)*(pB1*pA2-D)))
    }
    # comute upper and lower limits for the optimization routine 
    C1hat <- max(-pa1*pa2,-(1-pa1)*(1-pa2)); C2hat <- min(pa1*(1-pa2),(1-pa1)*pa2)
    # Find the MLE
    optim.run <- optim(0,ll,method="Brent", lower=C1hat, upper=C2hat, pA1=pa1,pA2=pa2, control=list(reltol=1e-15))
    # Estimate D and ensure that it is within its defined range
    D <- (optim.run$par)*(2*N/(2*N-1))
    D <- ifelse(D>=0,min(D,C2hat),max(D,C1hat))
    # Produce the estimates for r2 and Dprime
    Dprime <- D/ifelse(D<0, abs(C1hat), C2hat)
    r2 <- D^2/(prod(c(pa1,pa2,1-pa1,1-pa2)))
    return(c(D,Dprime,r2,pa1,pa2))
  }
}


### Likelihood function that adjusts for read depth (old approach)
estMLEadjustedLik <- function(SEQgeno,depth,initialV){

  ## check to see if the initial value starting values are not too close to zero
  if(initialV[1] > 0.999)
    initialV[1] = 0.999
  if(initialV[2] > 0.999)
    initialV[2] = 0.999
  
  ### Determine which individuals have reads at both SNPs
  indx <- which(apply(depth,1,function(x) all(x>=1)))
  N <- length(indx)

  # Compute the matrix of GBS probs conditional on the true genotypes
  nonNegDepth <- depth[indx,]
  K <- 1/(2^nonNegDepth)
  coefMat <- cbind(K,1-2*K,K[,1]*K[,2])

  ## Subset the data to include only individuals with genotypes at both SNPs
  adGeno <- SEQgeno[indx,]

  my.grid <- lapply(1:9, function(i) as.matrix(expand.grid(2:0,2:0))[i,])
  id <- lapply(my.grid,function(x) which(adGeno[,1]==x[1] & adGeno[,2]==x[2]))

  ## likelihood function for the GBS genotypes
  pstar.ll <- function(x){
    pA1=x[1]; pA2=x[2]; D=x[3]
    pB1 = 1-pA1; pB2 = 1-pA2
    C1 <- max(-pA1*pA2,-pB1*pB2); C2 <- min(pB1*pA2,pA1*pB2)
    if(!((C1 <= D) & (D <= C2))) return(NA)
    else{
      h1 <- pA1*pA2+D; h2 <- pA1*pB2-D; h3 <- pB1*pA2-D; h4 <- pB1*pB2+D
      p11 <- h1^2
      p21 <- 2*h1*h3
      p31 <- h3^2
      p12 <- 2*h1*h2
      p32 <- 2*h3*h4
      p13 <- h2^2
      p23 <- 2*h2*h4
      p33 <- h4^2
      p22 <- 2*h1*h4+2*h2*h3

      return(-(sum(log(p11+(coefMat[,2]*p12+coefMat[,1]*p21+coefMat[,5]*p22)[id[[1]]])) +
                 sum(log((coefMat[,3]*p21+coefMat[,3]*coefMat[,2]*p22)[id[[2]]])) +
                 sum(log(p31 + (coefMat[,1]*p21+coefMat[,2]*p32+coefMat[,5]*p22)[id[[3]]])) +
                 sum(log((coefMat[,4]*p12+coefMat[,4]*coefMat[,1]*p22)[id[[4]]])) +
                 sum(log((coefMat[,3]*coefMat[,4]*p22)[id[[5]]])) +
                 sum(log((coefMat[,4]*p32+coefMat[,4]*coefMat[,1]*p22)[id[[6]]])) +
                 sum(log(p13 + (coefMat[,2]*p12+coefMat[,1]*p23+coefMat[,5]*p22)[id[[7]]])) +
                 sum(log((coefMat[,3]*p23+coefMat[,3]*coefMat[,2]*p22)[id[[8]]])) +
                 sum(log(p33 + (coefMat[,2]*p32+coefMat[,1]*p23+coefMat[,5]*p22)[id[[9]]]))
      ))
    }
  }

  ## Compute MLE
  MLE <- optim(initialV,pstar.ll,method="Nelder-Mead", control=list(reltol=1e-20))$par
  D <- MLE[3]*(2*N/(2*N-1))

  ## Check that D is within its range
  C1hat <- max(-prod(MLE[1:2]),-prod(1-MLE[1:2])); C2hat <- min((1-MLE[1])*MLE[2],MLE[1]*(1-MLE[2]))
  D <- ifelse(D>=0,min(D,C2hat),max(D,C1hat))
  
  ## Compute the estimates of Dprime and r2
  Dprime <- D/ifelse(D<0, abs(C1hat), C2hat)
  r2 <- D^2/(prod(c(MLE[1:2],1-MLE[1:2])))
  # return the estimates
  return(c(D,Dprime,r2,MLE[1],MLE[2]))
}

### Likelihood function that adjusts for read depth
### and includes estimation of sequencing errors
estMLEadjustedLik2 <- function(depth_Ref,depth_Alt,initialV){
  
  pAB <- choose(depth_Ref+depth_Alt,depth_Ref) * (1/2)^(depth_Ref + depth_Alt)
  N <- length(apply(depth_Ref==0&depth_Alt==0,1,any))
  
  ## check to see if the initial value starting values are not too close to zero
  if(initialV[1] > 0.999)
    initialV[1] = 0.999
  if(initialV[2] > 0.999)
    initialV[2] = 0.999

  ## likelihood function
  
  ll_wrapper <- function(logit_x, depth_Ref, depth_Alt, pAB){
    pA1=inv.logit(logit_x[2])
    pA2=inv.logit(logit_x[3])
    ep =inv.logit(logit_x[4])
    r = 2*inv.logit(logit_x[1]) - 1
    D = r*sqrt(pA1*pA2*(1-pA1)*(1-pA2))
    return(ll(c(pA1,pA2,D,ep), depth_Ref, depth_Alt, pAB))
  }
  ll <- function(x, depth_Ref, depth_Alt, pAB){
    pA1=x[1]; pA2=x[2]; D=x[3]
    pB1 = 1-pA1; pB2 = 1-pA2
    C1 <- max(-pA1*pA2,-pB1*pB2); C2 <- min(pB1*pA2,pA1*pB2)
    if(!((C1 <= D) & (D <= C2))) return(NA)
    else{    
      ep = x[4]
      ## compute the probability matrices
      pAA <- choose(depth_Ref+depth_Alt,depth_Ref) * (1-ep)^depth_Ref * ep^depth_Alt
      pBB <- choose(depth_Ref+depth_Alt,depth_Ref) * (ep)^depth_Ref * (1-ep)^depth_Alt

      h1 <- pA1*pA2+D; h2 <- pA1*pB2-D; h3 <- pB1*pA2-D; h4 <- pB1*pB2+D
      p11 <- h1^2
      p21 <- 2*h1*h3
      p31 <- h3^2
      p12 <- 2*h1*h2
      p32 <- 2*h3*h4
      p13 <- h2^2
      p23 <- 2*h2*h4
      p33 <- h4^2
      p22 <- 2*h1*h4+2*h2*h3
      
      return(-(sum( log(
        pAA[,1]*pAA[,2]*p11 + pAA[,1]*pAB[,2]*p12 + pAA[,1]*pBB[,2]*p13 +
          pAB[,1]*pAA[,2]*p21 + pAB[,1]*pAB[,2]*p22 + pAB[,1]*pBB[,2]*p23 +
          pBB[,1]*pAA[,2]*p31 + pBB[,1]*pAB[,2]*p32 + pBB[,1]*pBB[,2]*p33
      )))  )
    }
  }
  ## Compute MLE
  MLE <- optim(logit(c(0.5,initialV[1:2],0.01)),ll_wrapper,
               method="Nelder-Mead", control=list(reltol=1e-20, maxit=2000), depth_Ref=depth_Ref, depth_Alt=depth_Alt, pAB=pAB)
  MLE2 <- optim(logit(c(0.5,initialV[1:2],0.0001)),ll_wrapper,
                method="Nelder-Mead", control=list(reltol=1e-20, maxit=2000), depth_Ref=depth_Ref, depth_Alt=depth_Alt, pAB=pAB)
  if(MLE$value < MLE2$value)
    MLE <- MLE$par
  else
    MLE <- MLE2$par
  pA1_hat = inv.logit(MLE[2])
  pA2_hat = inv.logit(MLE[3])
  D_hat <- (2*inv.logit(MLE[1])-1)*sqrt(pA1_hat*pA2_hat*(1-pA1_hat)*(1-pA2_hat))
  D_hat <- D_hat*(2*N/(2*N-1))
  ep_hat = inv.logit(MLE[4])
  ## Check that D is within its range
  C1hat <- max(-(pA1_hat*pA2_hat),-prod(1-c(pA1_hat,pA2_hat))); C2hat <- min((1-pA1_hat)*pA2_hat,pA1_hat*(1-pA2_hat))
  D_hat <- ifelse(D_hat>=0,min(D_hat,C2hat),max(D_hat,C1hat))
  
  ## Compute the estimates of Dprime and r2  
  Dprime <- D_hat/ifelse(D_hat<0, abs(C1hat), C2hat)
  r2 <- D_hat^2/(pA1_hat*pA2_hat*(1-pA1_hat)*(1-pA2_hat))
  # return the estimates
  return(c(D_hat,Dprime,r2,pA1_hat,pA2_hat,ep_hat))
}


#######################################
#######################################
####### First set of simulations
####### Comparing GUS-LD with Standard approach

## Function for implementing the simulation
GBS_LD_Sim <- function(pA1,pA2,D,epsilon,NoRuns,nInd){
  pB1 <- 1-pA1
  pB2 <- 1-pA2
  
  ## Check that the contraint on D is met
  
  C1 <- round(max(-pA1*pA2,-pB1*pB2),16); C2 <- round(min(pB1*pA2,pA1*pB2),16)
  if (!((C1 <= D) & (D <= C2))) stop('Error: User specified value of D is outside it constrained region') 
  
  ###### Calculate true values of the LD measures
  # r^2
  r2 <- D^2/(pA1*pA2*pB1*pB2)
  # D'
  Dcon <- ifelse(D<0, abs(C1), C2)
  Dprime <- D/(Dcon)
  
  ## Table of true haplotype probabilities                                                                                                                                                 
  haploP <- round(matrix(c(pA1*pA2+D,pA1*pB2-D,pB1*pA2-D,pB1*pB2+D), ncol=2,
                         dimnames=list(c('A1','B1'),c('A2','B2')), byrow=T),8)
  
  Depths <- c(1:5,7.5,10,15)                        # vector of mean depths
  Est.mat <- matrix(NA,nrow=length(Depths), ncol=10*3+2) # Matrix containing the monte carlo bias and variance of MLE's
  colnames(Est.mat) <- c(paste0(rep(c("E(D","E(D'","E(r2","E(pa1","E(pa2","Se(D","Se(D'","Se(r2","Se(pa1","Se(pa2"),3),").",
                              c(rep(1,10),rep(2,10),rep(3,10))),"E(ep)","Se(ep)")
  
  ### Define some values
  Dhat <-  Dprimehat <- r2hat <- pa1hat <- pa2hat <- numeric(NoRuns)     
  Dhat2 <- Dprimehat2 <- r2hat2 <- pa1hat2 <- pa2hat2 <- numeric(NoRuns)   
  Dhat3 <- Dprimehat3 <- r2hat3 <- pa1hat3 <- pa2hat3 <- ephat <- numeric(NoRuns)   
  
  Dtrue <- DprimeTrue <- r2True <- numeric(NoRuns*length(Depths))
  
  for(meanDepth in Depths){
    
    set.seed(42198+99*meanDepth+11*nInd-D*10000+pA1*199+pA2*223)
    
    for(run in 1:NoRuns){
      
      ## Generate the true genotypes
      obsH <- matrix(sample(c('AA','BA','AB','BB'),size=nInd*2,
                            prob=as.vector(haploP), replace=T), ncol=2)
      trueGeno <- t(apply(obsH,1, function(x) c(sum(substr(x,1,1) == 'A'),sum(substr(x,2,2) == 'A'))))
      depth <- matrix(rpois(nInd*2,meanDepth),ncol=2) 
      
      ##### Generate the GBS data from the true genotypes
      # Simulate the observed A alleles
      aCounts <- matrix(rbinom(nInd*2,depth,trueGeno/2),ncol=2)
      bCounts <- depth - aCounts
      aCountsFinal <- matrix(rbinom(nInd*2,aCounts,prob=1-epsilon),ncol=2) + matrix(rbinom(nInd*2,bCounts,prob=epsilon),ncol=2)
      bCountsFinal <- depth - aCountsFinal
      SEQgeno <- aCountsFinal/depth
      SEQgeno[which(SEQgeno^2-SEQgeno<0)] <- 0.5
      SEQgeno <- 2* SEQgeno  ## GBS genotype call
      GBSfreq <- colMeans(SEQgeno,na.rm=TRUE)/2
      
      ### Want to compute the estimates of the LD measure using the likelihood approach
      GBSgF <- table(factor(SEQgeno[,1],levels=0:2,labels=0:2),factor(SEQgeno[,2],levels=0:2,labels=0:2))
      GBSgF <- GBSgF[3:1,3:1]
      
      ### Compute the monte carlo estimate of the true variance 
      tabH <- table(factor(obsH,levels=c('AA','AB','BA','BB')))
      pa1 = sum(tabH[names(tabH) == 'AA'],tabH[names(tabH) == 'AB'])/(2*nInd)
      pa2 = sum(tabH[names(tabH) == 'AA'],tabH[names(tabH) == 'BA'])/(2*nInd)
      
      Dtrue <- tabH[names(tabH) == 'AA']/(2*nInd) - pa1*pa2
      DprimeTrue <- Dtrue/ifelse(Dtrue<0,min(pa1*pa2,(1-pa1)*(1-pa2)),  min(pa1*(1-pa2),(1-pa1)*pa2))
      r2True <- Dtrue^2/(pa1*pa2*(1-pa1)*(1-pa2))
      
      #### Find estimates of D' and r2 using the nieve approach
      unAdjEst <- estMLE(GBSgF,GBSfreq)
      Dhat[run] <- unAdjEst[1]
      Dprimehat[run] <- unAdjEst[2]
      r2hat[run] <- unAdjEst[3]
      pa1hat[run] <- unAdjEst[4]
      pa2hat[run] <- unAdjEst[5]
      
      #### Estimate D' and r2 using adjustments to the genotype frequencies
      AdjEst <- estMLEadjustedLik(SEQgeno,depth,c(GBSfreq,0))
      Dhat2[run] <- AdjEst[1]
      Dprimehat2[run] <- AdjEst[2]
      r2hat2[run] <- AdjEst[3]
      pa1hat2[run] <- AdjEst[4]
      pa2hat2[run] <- AdjEst[5]
      
      AdjEst2 <- estMLEadjustedLik2(aCountsFinal,bCountsFinal,c(GBSfreq,0,0))
      Dhat3[run] <- AdjEst2[1]
      Dprimehat3[run] <- AdjEst2[2]
      r2hat3[run] <- AdjEst2[3]
      pa1hat3[run] <- AdjEst2[4]
      pa2hat3[run] <- AdjEst2[5]
      ephat[run] <- AdjEst2[6]
    }
    # Sort out the phats for convience
    pa1hat <- pmax(pa1hat,1-pa1hat)
    pa1hat2 <- pmax(pa1hat2,1-pa1hat2)
    pa1hat3 <- pmax(pa1hat3,1-pa1hat3)
    pa2hat <- pmax(pa2hat,1-pa2hat)
    pa2hat2 <- pmax(pa2hat2,1-pa2hat2)
    pa2hat3 <- pmax(pa2hat3,1-pa2hat3)
    
    # Compute the Monte Carlo expectation for the LD measures
    E_D1 <- mean(Dhat, na.rm = T); E_D2 <- mean(Dhat2, na.rm = T); E_D3 <- mean(Dhat3, na.rm = T)
    E_Dprime1 <- mean(Dprimehat, na.rm = T); E_Dprime2 <- mean(Dprimehat2, na.rm = T); E_Dprime3 <- mean(Dprimehat3, na.rm = T)
    E_r21 <- mean(r2hat, na.rm = T); E_r22 <- mean(r2hat2, na.rm = T); E_r23 <- mean(r2hat3, na.rm = T)
    E_pa11 <- mean(pa1hat, na.rm = T); E_pa12 <- mean(pa1hat2, na.rm = T); E_pa13 <- mean(pa1hat3, na.rm = T)
    E_pa21 <- mean(pa2hat, na.rm = T); E_pa22 <- mean(pa2hat2, na.rm = T); E_pa23 <- mean(pa2hat3, na.rm = T)
    E_ep <- mean(ephat, na.rm = T)
    Se_D1 <- sqrt(mean((Dhat-E_D1)^2, na.rm = T)); Se_D2 <- sqrt(mean((Dhat2-E_D2)^2, na.rm = T)); Se_D3 <- sqrt(mean((Dhat3-E_D3)^2, na.rm = T))
    Se_Dprime1 <- sqrt(mean((Dprimehat-E_Dprime1)^2, na.rm = T)); Se_Dprime2 <- sqrt(mean((Dprimehat2-E_Dprime2)^2, na.rm = T)); Se_Dprime3 <- sqrt(mean((Dprimehat3-E_Dprime3)^2, na.rm = T))
    Se_r21 <- sqrt(mean((r2hat-E_r21)^2, na.rm = T)); Se_r22 <- sqrt(mean((r2hat2-E_r22)^2, na.rm = T)); Se_r23 <- sqrt(mean((r2hat3-E_r23)^2, na.rm = T))
    Se_pa1 <- sqrt(mean((pa1hat-E_pa11)^2, na.rm = T)); Se_pa12 <- sqrt(mean((pa1hat2-E_pa12)^2, na.rm = T)); Se_pa13 <- sqrt(mean((pa1hat3-E_pa13)^2, na.rm = T))
    Se_pa2 <- sqrt(mean((pa2hat-E_pa21)^2, na.rm = T)); Se_pa22 <- sqrt(mean((pa2hat2-E_pa22)^2, na.rm = T)); Se_pa23 <- sqrt(mean((pa2hat3-E_pa23)^2, na.rm = T))
    Se_ep <- sqrt(mean((ephat-E_ep)^2, na.rm = T))
    # Compute the Monte Carlo bias and variance of the MLEs
    Est.mat[which(meanDepth == Depths),] <- c(E_D1,E_Dprime1,E_r21,E_pa11,E_pa21,Se_D1,Se_Dprime1,Se_r21,Se_pa1,Se_pa2,
                                              E_D2,E_Dprime2,E_r22,E_pa12,E_pa22,Se_D2,Se_Dprime2,Se_r22,Se_pa12,Se_pa22,
                                              E_D3,E_Dprime3,E_r23,E_pa13,E_pa23,Se_D3,Se_Dprime3,Se_r23,Se_pa13,Se_pa23,E_ep,Se_ep)
    
  }
  write.table(Est.mat,
              file=paste0('SimData/GBS_Est_Mat_pA1_',pA1,'_pA2_',pA2,'_D',D,'_N',nInd,'Runs',NoRuns,'.txt',sep=''))
  write(Depths,file=paste0('SimData/GBS_depth_pA1_',pA1,'_pA2_',pA2,'_D',D,'_N',nInd,'Runs',NoRuns,'.txt',sep=''))
  write(c((mean(Dtrue-D)^2),mean((DprimeTrue-Dprime)^2),mean((r2True-r2)^2)),
        file=paste0('SimData/GBS_trueVar_pA1_',pA1,'_pA2_',pA2,'_D',D,'_N',nInd,'Runs',NoRuns,'.txt',sep=''))
}

stop("end source here")

## Run the simulations for various combination of pA1,pA2 and D.
for(D in c(-0.15,0,0.05,0.15,0.25))
  GBS_LD_Sim(0.5,0.5,D,0.01,10000,100)
for(D in c(0,0.05,0.1,0.125))
  GBS_LD_Sim(0.5,0.75,D,0.01,10000,100)
for(D in c(-0.01,0.03,0.06,0.09))
  GBS_LD_Sim(0.9,0.9,D,0.01,10000,100)




#### Now produce the plots
Nruns=10000; n=100; nMet = c(0,2)
for(sim in 1:3){
  if(sim == 1){ pA1=pA2=0.5; D=c(-0.15,0,0.05,0.15,0.25) }
  else if(sim == 2){ pA1=0.5; pA2=0.75; D=c(0,0.05,0.1,0.125) }
  else if(sim == 3){ pA1=0.9; pA2=0.9; D=c(-0.01,0.03,0.06,0.09) }
  
  Est.mat <- Depths <- trueVar <- vector(mode='list')

  for(i in 1:length(D)){
    Est.mat[[i]] <- as.matrix(read.table(paste0('SimData/GBS_Est_Mat_pA1_',pA1,'_pA2_',pA2,'_D',D[i],'_N',n,'Runs',Nruns,'.txt'),header=T))
    Depths[[i]] <- scan(paste0('SimData/GBS_depth_pA1_',pA1,'_pA2_',pA2,'_D',D[i],'_N',n,'Runs',Nruns,'.txt'))
    trueVar[[i]] <- scan(paste0('SimData/GBS_trueVar_pA1_',pA1,'_pA2_',pA2,'_D',D[i],'_N',n,'Runs',Nruns,'.txt'))
    ## remove depth 20
    Est.mat[[i]] <- Est.mat[[i]][-9,]
    Depths[[i]] <- Depths[[i]][-9]
  }
  
  #### Plots of D
  filename=paste0('./GBS_LD_sim_pA1_',pA1,'_pA2_',pA2,'_D',D[1],'-',D[length(D)],'_N',n,'Runs',Nruns)
  tiff(paste0(filename,'.tiff'), width=7.87*500, height=9.05*500, res=500, compression='lzw')
  layout(matrix(c(7,7,1:6),byrow=T,ncol=2), heights=c(1.3,5,5,5))
  par(mar=c(2.5,4.7,1.6,2.1),oma=c(3,0,0,0))
  
  ## Bias(D)
  plot((Est.mat[[1]][,1+nMet[1]*10]-D[[1]])~Depths[[1]], type='o', ylab="Bias(D)", xlab="", cex = 1.2, cex.lab = 1.4, cex.axis=1.4, xaxt='n', cex.axis=1.4,
       ylim=c(min(unlist(lapply(Est.mat, function(x) min(x[,1+nMet*10])))-D),max(unlist(lapply(Est.mat, function(x) max(x[,1+nMet*10])))-D)), lty=nMet[1]+1)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=3)
  mtext("A",side=3,cex=1, font=2, adj=0)
  for(j in nMet[-1]){
    lines(Est.mat[[1]][,1+j*10]-D[[1]]~Depths[[1]], col=1,lty=j+1)
    points(Est.mat[[1]][,1+j*10]-D[[1]]~Depths[[1]], col=1, cex = 1.2)
  }
  for(i in 2:length(D)){
    for(j in nMet){
      lines(Est.mat[[i]][,1+j*10]-D[[i]]~Depths[[i]],col=i,lty=j+1)
      points(Est.mat[[i]][,1+j*10]-D[[i]]~Depths[[i]], col=i,pch=i, cex = 1.2)
    }
  }
  ## Var(D)
  plot(Est.mat[[1]][,6+nMet[1]*10]~Depths[[1]], type='o', ylab="SE(D)", xlab="", cex = 1.2, cex.lab = 1.4, xaxt='n', cex.axis=1.4,
       ylim=c(0,max(unlist(lapply(Est.mat, function(x) max(x[,6+nMet*10]))))), lty=nMet[1]+1)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=2)
  mtext("B",side=3,cex=1, font=2, adj=0)
  for(j in nMet[-1]){
    lines(Est.mat[[1]][,6+j*10]~Depths[[1]], col=1,lty=j+1)
    points(Est.mat[[1]][,6+j*10]~Depths[[1]], col=1, cex = 1.2)
  }
  for(i in 2:length(D)){
    for(j in nMet){
      lines(Est.mat[[i]][,6+j*10]~Depths[[i]],col=i,lty=j+1)
      points(Est.mat[[i]][,6+j*10]~Depths[[i]], col=i,pch=i, cex = 1.2)
    }
  }
  
  #### Plots of Dprime
  ## Bias(Dprime)
  pB1=1-pA1; pB2=1-pA2
  C1 <- round(max(-pA1*pA2,-pB1*pB2),16); C2 <- round(min(pB1*pA2,pA1*pB2),16)
  Dprime = D/ifelse(D<0,abs(C1),C2)
  plot(Est.mat[[1]][,2+nMet[1]*10]-Dprime[[1]]~Depths[[1]], type='o', ylab="Bias(D')", xlab="", cex=1.2, cex.lab = 1.4, xaxt='n', cex.axis=1.4,
       ylim=c(min(unlist(lapply(Est.mat, function(x) min(x[,2+nMet*10])))-Dprime),max(unlist(lapply(Est.mat, function(x) max(x[,2+nMet*10])))-Dprime)), lty=nMet[1]+1)
  abline(h=0, lty=2)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  mtext("C",side=3,cex=1, font=2, adj=0)
  for(j in nMet[-1]){
    lines(Est.mat[[1]][,2+j*10]-Dprime[[1]]~Depths[[1]], col=1,lty=j+1)
    points(Est.mat[[1]][,2+j*10]-Dprime[[1]]~Depths[[1]], col=1, cex = 1.2)
  }
  for(i in 2:length(D)){
    for(j in nMet){
      lines(Est.mat[[i]][,2+j*10]-Dprime[[i]]~Depths[[i]],col=i,lty=j+1)
      points(Est.mat[[i]][,2+j*10]-Dprime[[i]]~Depths[[i]], col=i,pch=i, cex = 1.2)
    }
  }
  ## Var(Dprime)
  plot(Est.mat[[1]][,7+nMet[1]*10]~Depths[[1]], type='o', ylab="SE(D')", xlab="", cex=1.2, cex.lab = 1.4, xaxt='n', cex.axis=1.4,
       ylim=c(0,max(unlist(lapply(Est.mat, function(x) max(x[,7+nMet*10]))))), lty=nMet[1]+1)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=2)
  mtext("D",side=3,cex=1, font=2, adj=0)
  for(j in nMet[-1]){
    lines(Est.mat[[1]][,7+j*10]~Depths[[1]], col=1,lty=j+1)
    points(Est.mat[[1]][,7+j*10]~Depths[[1]], col=1, cex = 1.2)
  }
  for(i in 2:length(D)){
    for(j in nMet){
      lines(Est.mat[[i]][,7+j*10]~Depths[[i]],col=i,lty=j+1)
      points(Est.mat[[i]][,7+j*10]~Depths[[i]], col=i,pch=i, cex = 1.2)
    }
  }
  
  #### Plots of r2
  ## Bias(r2)
  r2=D^2/(pA1*pA2*pB1*pB2)
  plot(Est.mat[[1]][,3+nMet[1]*10]-r2[1]~Depths[[1]], type='o', ylab=expression(Bias(r^2)), xlab='Mean Depth', cex=1.2, cex.lab = 1.4, xaxt='n', cex.axis=1.4,
       ylim=c(min(unlist(lapply(Est.mat, function(x) min(x[,3+nMet*10])))-r2),max(unlist(lapply(Est.mat, function(x) max(x[,3+nMet*10])))-r2)), xpd=NA, lty=nMet[1]+1)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=3)
  mtext("E",side=3,cex=1, font=2, adj=0)
  for(j in nMet[-1]){
    lines(Est.mat[[1]][,3+j*10]-r2[1]~Depths[[1]], col=1,lty=j+1)
    points(Est.mat[[1]][,3+j*10]-r2[1]~Depths[[1]], col=1, cex = 1.2)
  }
  for(i in 2:length(D)){
    for(j in nMet){
      lines(Est.mat[[i]][,3+j*10]-r2[i]~Depths[[i]],col=i,lty=j+1)
      points(Est.mat[[i]][,3+j*10]-r2[i]~Depths[[i]], col=i,pch=i, cex = 1.2)
    }
  }
  ## Var(r2)
  plot(Est.mat[[1]][,9]~Depths[[1]], type='o', ylab=expression(SE(r^2)), xlab='Mean Depth', cex = 1.2, cex.lab = 1.4, xaxt='n', cex.axis=1.4,
       ylim=c(0,max(unlist(lapply(Est.mat, function(x) max(x[,8+nMet*10]))))), xpd=NA, lty=nMet[1]+1)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=2)
  mtext("F",side=3,cex=1, font=2, adj=0)
  for(j in nMet[-1]){
    lines(Est.mat[[1]][,8+j*10]~Depths[[1]], col=1,lty=j+1)
    points(Est.mat[[1]][,8+j*10]~Depths[[1]], col=1, cex = 1.2)
  }
  for(i in 2:length(D)){
    for(j in nMet){
      lines(Est.mat[[i]][,8+j*10]~Depths[[i]],col=i,lty=j+1)
      points(Est.mat[[i]][,8+j*10]~Depths[[i]], col=i,pch=i, cex = 1.2)
    }
  }
  
  ## for pa1=pa2=0.5
  par(mar=c(0,0,0,0))
  plot(NULL,NULL,ylim=c(0,1),xlim=c(0,1), xaxt='n', yaxt='n', bty='n')
  nD <- length(D)
  legend('center',legend=rep("  ",nD), lty=1,pch=1:nD, col=1:nD, xpd=NA, horiz = T, bty='n',cex=1.4)
  text(0.22,0.55,"Standard", cex=1.4)
  legend('bottom',legend=rep("  ",nD), lty=2,pch=1:nD, col=1:nD, xpd=NA, horiz = T, bty='n',cex=1.4)
  ## for pa1=0.5, pa2=0.75
  if(sim == 1| sim == 3)
    text(0.48,0.8,paste(format(D,2),collapse = "      "),cex=1.4)
  else if(sim == 4)
    text(0.48,0.8,paste(formatC(D,2),collapse = "   "),cex=1.4)
  ## for pa1=0.9, pa2=0.9
  else if (sim == 2)
    text(0.48,0.8,paste(format(D,2),collapse = "    "),cex=1.4)
  text(0.22,0.25,"GUS-LD",cex=1.4)
  text(0.22,0.8,"D",cex=1.4)
  rect(0.16,0,0.68,1)

  dev.off()
}

## plot the variance of the minor allele frequencies
Nruns=10000; n=100; nMet = c(0,2)

filename=paste0('./GBS_LD_sim_allele_freq')
tiff(paste0(filename,'.tiff'), width=7.87*500, height=9.05*500, res=500, compression='lzw')
layout(matrix(c(1:6),byrow=T,ncol=2), heights=c(5,5,5))
par(mar=c(2.5,4.7,1.6,2.1),oma=c(3,0,0,0))

for(sim in 1:3){
  if(sim == 1){ pA1=pA2=0.5; D=c(-0.15,0,0.05,0.15,0.25) }
  else if(sim == 2){ pA1=0.5; pA2=0.75; D=c(0,0.05,0.1,0.125) }
  else if(sim == 3){ pA1=0.9; pA2=0.9; D=c(-0.01,0.03,0.06,0.09) }

  Est.mat <- Depths <- trueVar <- vector(mode='list')
  
  for(i in 1:length(D)){
    Est.mat[[i]] <- as.matrix(read.table(paste0('SimData/GBS_Est_Mat_pA1_',pA1,'_pA2_',pA2,'_D',D[i],'_N',n,'Runs',Nruns,'.txt'),header=T))
    Depths[[i]] <- scan(paste0('SimData/GBS_depth_pA1_',pA1,'_pA2_',pA2,'_D',D[i],'_N',n,'Runs',Nruns,'.txt'))
    trueVar[[i]] <- scan(paste0('SimData/GBS_trueVar_pA1_',pA1,'_pA2_',pA2,'_D',D[i],'_N',n,'Runs',Nruns,'.txt'))
    ## remove depth 20
    Est.mat[[i]] <- Est.mat[[i]][-9,]
    Depths[[i]] <- Depths[[i]][-9]
  }
  
  ## Var(pA1)
  plot((Est.mat[[1]][,9+nMet[1]*10])~Depths[[1]], type='l', ylab=expression(SE(italic(p[A[1]]))), xlab=ifelse(sim==3,'Mean Depth',""), cex = 1.2, cex.lab = 1.4, cex.axis=1.4, xaxt='n', cex.axis=1.4,
       ylim=c(0,max(unlist(lapply(Est.mat, function(x) max(x[,9+nMet*10]))))), lty=nMet[1]+1)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=3)
  mtext(switch(sim,"A","C","E"),side=3,cex=1, font=2, adj=0)
  for(j in nMet[-1]){
    lines(Est.mat[[1]][,9+j*10]~Depths[[1]], col=1,lty=j+1)
    #points(Est.mat[[1]][,9+j*10]~Depths[[1]], col=1, cex = 1.2)
  }
  for(i in 2:length(D)){
    for(j in nMet){
      lines(Est.mat[[i]][,9+j*10]~Depths[[i]],col=i,lty=j+1)
      #points(Est.mat[[i]][,9+j*10]~Depths[[i]], col=i,pch=i, cex = 1.2)
    }
  }
  ## Var(pA2)
  plot((Est.mat[[1]][,10+nMet[1]*10])~Depths[[1]], type='l', ylab=expression(SE(italic(p[A[2]]))), xlab=ifelse(sim==3,'Mean Depth',""), cex = 1.2, cex.lab = 1.4, cex.axis=1.4, xaxt='n', cex.axis=1.4,
       ylim=c(0,max(unlist(lapply(Est.mat, function(x) max(x[,10+nMet*10]))))), lty=nMet[1]+1)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=3)
  mtext(switch(sim,"B","D","F"),side=3,cex=1, font=2, adj=0)
  for(j in nMet[-1]){
    lines(Est.mat[[1]][,10+j*10]~Depths[[1]], col=1,lty=which(j == nMet))
    #points(Est.mat[[1]][,10+j*10]~Depths[[1]], col=1, cex = 1.2)
  }
  for(i in 2:length(D)){
    for(j in nMet){
      lines(Est.mat[[i]][,10+j*10]~Depths[[i]],col=i,lty=which(j == nMet))
      #points(Est.mat[[i]][,10+j*10]~Depths[[i]], col=i,pch=i, cex = 1.2)
    }
  }
  nD <- length(D)
  legend('topright', legend=c(rep("",2*nD),D), lty=c(rep(1,nD),rep(2,nD),rep(NA,nD)), col=c(rep(1:nD,2),rep(NA,nD)), xpd=NA, ncol=3,
          x.intersp=-2,title=paste(c("Std  ","GUS","D    "),collapse="    "),y.intersp=0.8)
}
dev.off()

filename=paste0('./GBS_LD_sim_epsilon')
tiff(paste0(filename,'.tiff'), width=7.87*500, height=9.05*500, res=500, compression='lzw')
layout(matrix(c(1:6),byrow=T,ncol=2), heights=c(5,5,5))
par(mar=c(2.5,4.7,1.6,2.1),oma=c(3,0,0,0))

for(sim in 1:3){
  if(sim == 1){ pA1=pA2=0.5; D=c(-0.15,0,0.05,0.15,0.25); ep=0.01 }
  else if(sim == 2){ pA1=0.5; pA2=0.75; D=c(0,0.05,0.1,0.125); ep=0.01 }
  else if(sim == 3){ pA1=0.9; pA2=0.9; D=c(-0.01,0.03,0.06,0.09); ep=0.01 }

  Est.mat <- Depths <- trueVar <- vector(mode='list')
  
  for(i in 1:length(D)){
    Est.mat[[i]] <- as.matrix(read.table(paste0('SimData/GBS_Est_Mat_pA1_',pA1,'_pA2_',pA2,'_D',D[i],'_N',n,'Runs',Nruns,'.txt'),header=T))
    Depths[[i]] <- scan(paste0('SimData/GBS_depth_pA1_',pA1,'_pA2_',pA2,'_D',D[i],'_N',n,'Runs',Nruns,'.txt'))
    trueVar[[i]] <- scan(paste0('SimData/GBS_trueVar_pA1_',pA1,'_pA2_',pA2,'_D',D[i],'_N',n,'Runs',Nruns,'.txt'))
    ## remove depth 20
    Est.mat[[i]] <- Est.mat[[i]][-9,]
    Depths[[i]] <- Depths[[i]][-9]
  }
  
  ## Bias(epsilon)
  plot((Est.mat[[1]][,31])-ep~Depths[[1]], type='o', ylab="Bias(\u03b5)", xlab=ifelse(sim==3,'Mean Depth',""), cex = 1.2, cex.lab = 1.4, cex.axis=1.4, xaxt='n', cex.axis=1.4,
       ylim=c(min(unlist(lapply(Est.mat, function(x) min(x[,31]-ep)))),max(unlist(lapply(Est.mat, function(x) max(x[,31]-ep))))),, xpd=NA)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=3)
  mtext(switch(sim,"A","C","E"),side=3,cex=1, font=2, adj=0)
  for(i in 2:length(D)){
    lines(Est.mat[[i]][,31]-ep~Depths[[i]],col=i)
    points(Est.mat[[i]][,31]-ep~Depths[[i]], col=i,pch=i, cex = 1.2)
  }
  nD <- length(D)
  legend("topright",legend=D,xpd=NA, horiz = F, cex=1.4, col=1:nD, lty=1,pch=1:nD,title="D")
  ## Var(epsilon)
  plot((Est.mat[[1]][,32])~Depths[[1]], type='o', ylab="SE(\u03b5)", xlab=ifelse(sim==3,'Mean Depth',""), cex = 1.2, cex.lab = 1.4, cex.axis=1.4, xaxt='n', cex.axis=1.4,
       ylim=c(0,max(unlist(lapply(Est.mat, function(x) max(x[,32]))))), xpd=NA)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=3)
  mtext(switch(sim,"B","D","F"),side=3,cex=1, font=2, adj=0)
  for(i in 2:length(D)){
    lines(Est.mat[[i]][,32]~Depths[[i]],col=i)
    points(Est.mat[[i]][,32]~Depths[[i]], col=i,pch=i, cex = 1.2)
  }
  nD <- length(D)
  legend("topright",legend=D,xpd=NA, horiz = F, cex=1.4, col=1:nD, lty=1,pch=1:nD,title="D")
}
dev.off()





################################################
################################################
##### Second set of simulations
##### Examing the sequencing effort

GBS_LD_Sim_fixSeq <- function(pA1,pA2,D,epsilon,NoRuns,cost){
  pB1 <- 1-pA1
  pB2 <- 1-pA2
  
  ## Check that the contraint on D is met
  
  C1 <- round(max(-pA1*pA2,-pB1*pB2),16); C2 <- round(min(pB1*pA2,pA1*pB2),16)
  if (!((C1 <= D) & (D <= C2))) stop('Error: User specified value of D is outside it constrained region') 
  
  ###### Calculate true values of the LD measures
  # r^2
  r2 <- D^2/(pA1*pA2*pB1*pB2)
  # D'
  Dcon <- ifelse(D<0, abs(C1), C2)
  Dprime <- D/(Dcon)
  
  ## Table of true haplotype probabilities                                                                                                                                                 
  haploP <- round(matrix(c(pA1*pA2+D,pA1*pB2-D,pB1*pA2-D,pB1*pB2+D), ncol=2,
                         dimnames=list(c('A1','B1'),c('A2','B2')), byrow=T),8)
  
  nInd = c(300,150,100,75,60,40,30,20)
  Est.mat <- data.frame(matrix(nrow=length(nInd),ncol=34))
  colnames(Est.mat) <- c('Depth',rep(c('Bias(D)','Sd(D)','MSE(D)','Bias(Dprime)','Sd(Dprime)',
                                     'MSE(Dprime)','Bias(r2)','Sd(r2)','MSE(r2)','Bias(pa1)','Sd(pa1)','MSE(pa1)',
                                     'Bias(pa2)','Sd(pa2)','MSE(pa2)'),2),'Bias(\u03b5)','Sd(\u03b5)','MSE(\u03b5)')
  
  ### Define some values
  Dhat <-  Dprimehat <- r2hat <- pa1hat <- pa2hat <- numeric(NoRuns)     
  Dhat2 <-  Dprimehat2 <- r2hat2 <- pa1hat2 <- pa2hat2 <- ephat <- numeric(NoRuns)     
  
  Dtrue <- DprimeTrue <- r2True <- numeric(NoRuns*length(nInd))
  
  for(n in nInd){
    
    meanDepth = cost/(n*2) 
    set.seed(42198+99*meanDepth+11*n-D*10000+pA1*199+pA2*223)
    
    for(run in 1:NoRuns){

      ## Generate the true genotypes
      obsH <- matrix(sample(c('AA','BA','AB','BB'),size=n*2,
                            prob=as.vector(haploP), replace=T), ncol=2)
      trueGeno <- t(apply(obsH,1, function(x) c(sum(substr(x,1,1) == 'A'),sum(substr(x,2,2) == 'A'))))
      depth <- matrix(rpois(n*2,meanDepth),ncol=2) 
      
      ##### Generate the GBS data from the true genotypes
      # Simulate the observed A alleles
      aCounts <- matrix(rbinom(n*2,depth,trueGeno/2),ncol=2)
      bCounts <- depth - aCounts
      aCountsFinal <- matrix(rbinom(n*2,aCounts,prob=1-epsilon),ncol=2) + matrix(rbinom(n*2,bCounts,prob=epsilon),ncol=2)
      bCountsFinal <- depth - aCountsFinal
      SEQgeno <- aCountsFinal/depth
      SEQgeno[which(SEQgeno^2-SEQgeno<0)] <- 0.5
      SEQgeno <- 2* SEQgeno  ## GBS genotype call
      GBSfreq <- colMeans(SEQgeno,na.rm=TRUE)/2
      
      ### Want to compute the estimates of the LD measure using the likelihood approach
      GBSgF <- table(factor(SEQgeno[,1],levels=0:2,labels=0:2),factor(SEQgeno[,2],levels=0:2,labels=0:2))
      GBSgF <- GBSgF[3:1,3:1]
      
      ### Compute the monte carlo estimate of the true variance 
      tabH <- table(factor(obsH,levels=c('AA','AB','BA','BB')))
      pa1 = sum(tabH[names(tabH) == 'AA'],tabH[names(tabH) == 'AB'])/(2*n)
      pa2 = sum(tabH[names(tabH) == 'AA'],tabH[names(tabH) == 'BA'])/(2*n)
      
      Dtrue <- tabH[names(tabH) == 'AA']/(2*n) - pa1*pa2
      DprimeTrue <- Dtrue/ifelse(Dtrue<0,min(pa1*pa2,(1-pa1)*(1-pa2)),  min(pa1*(1-pa2),(1-pa1)*pa2))
      r2True <- Dtrue^2/(pa1*pa2*(1-pa1)*(1-pa2))
      
      #### Find estimates of D' and r2 using the nieve approach
      unAdjEst <- estMLE(GBSgF,GBSfreq)
      Dhat[run] <- unAdjEst[1]
      Dprimehat[run] <- unAdjEst[2]
      r2hat[run] <- unAdjEst[3]
      pa1hat[run] <- unAdjEst[4]
      pa2hat[run] <- unAdjEst[5]
      
      AdjEst2 <- estMLEadjustedLik2(aCountsFinal,bCountsFinal,c(GBSfreq,0,0))
      Dhat2[run] <- AdjEst2[1]
      Dprimehat2[run] <- AdjEst2[2]
      r2hat2[run] <- AdjEst2[3]
      pa1hat2[run] <- AdjEst2[4]
      pa2hat2[run] <- AdjEst2[5]
      ephat[run] <- AdjEst2[6]
      
    }
    # Compute the Monte Carlo expectation for the LD measures
    E_D1 <- mean(Dhat, na.rm = T); E_D2 <- mean(Dhat2, na.rm = T)
    E_Dprime1 <- mean(Dprimehat, na.rm = T); E_Dprime2 <- mean(Dprimehat2, na.rm = T)
    E_r21 <- mean(r2hat, na.rm = T); E_r22 <- mean(r2hat2, na.rm = T)
    E_pa11 <- mean(pa1hat, na.rm = T); E_pa12 <- mean(pa1hat2, na.rm = T)
    E_pa21 <- mean(pa2hat, na.rm = T); E_pa22 <- mean(pa2hat2, na.rm = T)
    E_ep <- mean(ephat, na.rm = T)
    Var_D1 <- sqrt(mean((Dhat-E_D1)^2, na.rm = T)); Var_D2 <- sqrt(mean((Dhat2-E_D2)^2, na.rm = T))
    Var_Dprime1 <- mean((Dprimehat-E_Dprime1)^2, na.rm = T); Var_Dprime2 <- mean((Dprimehat2-E_Dprime2)^2, na.rm = T)
    Var_r21 <- mean((r2hat-E_r21)^2, na.rm = T); Var_r22 <- mean((r2hat2-E_r22)^2, na.rm = T)
    Var_pa11 <- mean((pa1hat-E_pa11)^2, na.rm = T); Var_pa12 <- mean((pa1hat2-E_pa12)^2, na.rm = T)
    Var_pa21 <- mean((pa2hat-E_pa21)^2, na.rm = T); Var_pa22 <- mean((pa2hat2-E_pa22)^2, na.rm = T)
    Var_ep <- mean((ephat-E_ep)^2, na.rm = T)
    # Compute the Monte Carlo bias and variance of the MLEs
    Est.mat[which(n == nInd),] <- c(meanDepth,
                                              E_D1-D, Var_D1, Var_D1 + (E_D1-D)^2,
                                              E_Dprime1-Dprime, Var_Dprime1, Var_Dprime1 + (E_Dprime1-Dprime)^2,
                                              E_r21-r2, Var_r21, Var_r21 + (E_r21-r2)^2,
                                              E_pa11-pA1, Var_pa11, Var_pa11 + (E_pa11-pA1)^2,
                                              E_pa21-pA2, Var_pa21, Var_pa21 + (E_pa21-pA2)^2,
                                              ## GUS-LD
                                              E_D2-D, Var_D2, Var_D2 + (E_D2-D)^2,
                                              E_Dprime2-Dprime, Var_Dprime2, Var_Dprime2 + (E_Dprime2-Dprime)^2,
                                              E_r22-r2, Var_r22, Var_r22 + (E_r22-r2)^2,
                                              E_pa12-pA1, Var_pa12, Var_pa12 + (E_pa12-pA1)^2,
                                              E_pa22-pA2, Var_pa22, Var_pa22 + (E_pa22-pA2)^2,
                                              E_ep-epsilon, Var_ep, Var_ep + (E_ep-epsilon)^2)

  }
  write.table(Est.mat,
              file=paste0('SimData/GBS_fixSeq_pA1_',pA1,'_pA2_',pA2,'_D',D,'_cost',cost,'Runs',NoRuns,'.txt',sep=''))
  write(c((mean(Dtrue-D)^2),mean((DprimeTrue-Dprime)^2),mean((r2True-r2)^2)),
        file=paste0('SimData/GBS_trueVar_fixSeq_pA1_',pA1,'_pA2_',pA2,'_D',D,'_cost',cost,'Runs',NoRuns,'.txt',sep=''))
}

for(D in c(-0.15,0,0.05,0.15,0.25))
  GBS_LD_Sim_fixSeq(0.5,0.5,D,0.01,10000,600)
for(D in c(0,0.05,0.1,0.125))
  GBS_LD_Sim_fixSeq(0.5,0.75,D,0.01,10000,600)
for(D in c(-0.01,0.03,0.06,0.09))
  GBS_LD_Sim_fixSeq(0.9,0.9,D,0.01,10000,600)


#### Now produce the plots
Nruns=10000; cost=600; nMet=c(0,1)
for(sim in 1:3){
  if(sim == 1){ pA1=pA2=0.5; D=c(-0.15,0,0.05,0.15,0.25) }
  else if(sim == 2){ pA1=0.5; pA2=0.75; D=c(0,0.05,0.1,0.125) }
  else if(sim == 3){ pA1=0.9; pA2=0.9; D=c(-0.01,0.03,0.06,0.09) }

  Est.mat <- trueVar <- vector(mode='list')
  
  for(i in 1:length(D)){
    Est.mat[[i]] <- as.matrix(read.table(paste0('SimData/GBS_fixSeq_pA1_',pA1,'_pA2_',pA2,'_D',D[i],'_cost',cost,'Runs',Nruns,'.txt'),header=T))
    trueVar[[i]] <- scan(paste0('SimData/GBS_trueVar_fixSeq_pA1_',pA1,'_pA2_',pA2,'_D',D[i],'_cost',cost,'Runs',Nruns,'.txt'))
  }
  
  #### Plots of D
  filename=paste0('./GBS_LD_sim_pA1_',pA1,'_pA2_',pA2,'_D',D[1],'-',D[length(D)],'_cost',cost,'Runs',Nruns)
  tiff(paste0(filename,'.tiff'), width=3.9*500, height=9.05*500, res=500, compression='lzw')
  layout(matrix(c(4,1:3),byrow=T,ncol=1), heights=c(1.3,5,5,5))
  par(mar=c(2.5,4.7,1.6,2.1),oma=c(3,0,0,0))
  
  ## MSE(D)
  plot((Est.mat[[1]][,3]^2+(Est.mat[[1]][,2])^2)~Est.mat[[1]][,1], type='o', ylab="MSE(D)", xlab="", cex = 1.2, cex.lab = 1.4, cex.axis=1.4, xaxt='n', cex.axis=1.4,
       ylim=c(0,min(0.08,max(unlist(lapply(Est.mat, function(x) max((x[,3+15*nMet]^2+(x[,2+15*nMet])^2))))))), lty=1)
       #ylim=c(0,0.01), lty=1)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=3)
  mtext("A",side=3,cex=1, font=2, adj=0)
  for(j in nMet[-1]){
    lines((Est.mat[[1]][,3+15*j]^2+(Est.mat[[1]][,2+15*j])^2)~Est.mat[[1]][,1], col=1,lty=which(j == nMet))
    points((Est.mat[[1]][,3+15*j]^2+(Est.mat[[1]][,2+15*j])^2)~Est.mat[[1]][,1], col=1, cex = 1.2)
  }
  for(i in 2:length(D)){
    for(j in nMet){
      lines((Est.mat[[i]][,3+15*j]^2+(Est.mat[[i]][,2+15*j])^2)~Est.mat[[i]][,1],col=i,lty=j+1)
      points((Est.mat[[i]][,3+15*j]^2+(Est.mat[[i]][,2+15*j])^2)~Est.mat[[i]][,1], col=i,pch=i, cex = 1.2)
    }
  }
  
  #### Plots of Dprime
  ## MSE(Dprime)
  plot((Est.mat[[1]][,6]+(Est.mat[[1]][,5])^2)~Est.mat[[1]][,1], type='o', ylab="MSE(D')", xlab="", cex = 1.2, cex.lab = 1.4, cex.axis=1.4, xaxt='n', cex.axis=1.4,
       ylim=c(0,max(unlist(lapply(Est.mat, function(x) max(x[,7+nMet*15]))))), lty=1)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=3)
  mtext("B",side=3,cex=1, font=2, adj=0)
  for(j in nMet[-1]){
    lines((Est.mat[[1]][,6+15*j]+(Est.mat[[1]][,5+15*j])^2)~Est.mat[[1]][,1], col=1,lty=which(j == nMet))
    points((Est.mat[[1]][,6+15*j]+(Est.mat[[1]][,5+15*j])^2)~Est.mat[[1]][,1], col=1, cex = 1.2)
  }
  for(i in 2:length(D)){
    for(j in nMet){
      lines(Est.mat[[i]][,7+j*15]~Est.mat[[i]][,1],col=i,lty=j+1)
      points(Est.mat[[i]][,7+j*15]~Est.mat[[i]][,1], col=i,pch=i, cex = 1.2)
    }
  }
  
  #### Plots of r2
  ## MSE(r2)
  plot(Est.mat[[1]][,10]~Est.mat[[1]][,1], type='o', ylab=expression(MSE(r^2)), xlab="Mean Depth", cex = 1.2, cex.lab = 1.4, cex.axis=1.4, xaxt='n', cex.axis=1.4,
       ylim=c(0,min(0.12,max(unlist(lapply(Est.mat, function(x) max(x[,10+nMet*15])))))), lty=1, xpd=NA)
  #     ylim=c(0,0.1), lty=1)
  axis(side=1,at=c(1,5,10,15),cex.axis=1.4)
  abline(h=0, lty=3)
  mtext("C",side=3,cex=1, font=2, adj=0)
  for(j in nMet[-1]){
    lines(Est.mat[[1]][,10+j*15]~Est.mat[[1]][,1], col=1,lty=which(j == nMet))
    points(Est.mat[[1]][,10+j*15]~Est.mat[[1]][,1], col=1, cex = 1.2)
  }
  for(i in 2:length(D)){
    for(j in nMet){
      lines(Est.mat[[i]][,10+j*15]~Est.mat[[i]][,1],col=i,lty=j+1)
      points(Est.mat[[i]][,10+j*15]~Est.mat[[i]][,1], col=i,pch=i, cex = 1.2)
    }
  }
  ## for pa1=pa2=0.5
  par(mar=c(0,0,0,0))
  plot(NULL,NULL,ylim=c(0,1),xlim=c(0,1), xaxt='n', yaxt='n', bty='n')
  nD <- length(D)
  ## for pa1=0.5, pa2=0.75
  if(sim %in% c(2:4)){
    if (sim == 2)
      text(0.67,0.8,paste(format(D,2),collapse = "     "),cex=1.4)
    else if(sim == 4)
      text(0.66,0.8,paste(formatC(D,2),collapse = "    "),cex=1.4)
    else
      text(0.66,0.8,paste(format(D,2),collapse = "      "),cex=1.4)
    legend('right',legend=rep("   ",nD), lty=1,pch=1:nD, col=1:nD, xpd=NA, horiz = T, bty='n',cex=1.4)
    text(0.22,0.55,"Standard", cex=1.4)
    legend('bottomright',legend=rep("   ",nD), lty=2,pch=1:nD, col=1:nD, xpd=NA, horiz = T, bty='n',cex=1.4)
  }
  else if (sim == 1){
    text(0.65,0.8,paste(format(D,2),collapse = "     "),cex=1.4)
    legend('right',legend=rep(" ",nD), lty=1,pch=1:nD, col=1:nD, xpd=NA, horiz = T, bty='n',cex=1.4)
    text(0.22,0.55,"Standard", cex=1.4)
    legend('bottomright',legend=rep(" ",nD), lty=2,pch=1:nD, col=1:nD, xpd=NA, horiz = T, bty='n',cex=1.4)
  }
  text(0.22,0.25,"GUS-LD",cex=1.4)
  text(0.22,0.8,"D",cex=1.4)
  rect(0.08,0,1,1)

  dev.off()
}




